#include "shellcmd.h"
/**
 * Appends parameters to atd syntax from aos_to_atsyntax.
 * @param cmd - array of strings containing command, last element must be
 *              NULL;
 * @param cmdSize - size of command array
 * @return None
 */
void executeCmd(char ** cmd, uint32_t cmdSize){

 pid_t pid;
 pid_t child = 0;
 pid_t error = -1;
 
 fflush(stdout);
 
 if(DEBUG_SHELLCMD)printf("Entering fork\n\r");

 pid = forkProc();
 if(pid != error){
    if(pid != child){ //parent process
        if(DEBUG_SHELLCMD)printf("parent wait %d\n\r", getpid());
        waitProc(pid); // wait for child process
        if(DEBUG_SHELLCMD)printf("parent done wait\n\r");
         }
    else { //child process
        if(DEBUG_SHELLCMD)fprintf(stderr,"in child %d\n\r", getpid());
        char *redirectLoc;
        if((redirectLoc = checkRedirect(cmd, cmdSize)) != NULL){
            executeRedirect(cmd, cmdSize, redirectLoc);
            }
        else{
            execute(cmd, cmdSize);
            exit(1);
            }
        }

    }
}

/**
 * Wrapper for forking a new process
 * @return process ID of new process
 */
pid_t forkProc(void){
    pid_t pid;
    if((pid = fork()) == -1){ perror("error fork\n\r");}
    return(pid);
}

/**
 * Wrapper for waiting on child process
 * @param pid - process ID of child that is being waited on
 * @return none
 */
void waitProc(pid_t pid){
    int status, retVal;
        if((retVal = waitpid(pid, &status, 0) ) > 0) {
            if(DEBUG_SHELLCMD){
                printf("Parent: Child done.\n");
                printf("Return value: %d\n", retVal);
                printf("Status:       %d\n", status);
                printf("WIFSTOPPED:   %d\n", WIFSTOPPED(status));
                printf("WIFSIGNALED:  %d\n", WIFSIGNALED(status));
                printf("WIFEXITED:    %d\n", WIFEXITED(status));
                printf("WEXITSTATUS:  %d\n", WEXITSTATUS(status));
                printf("WTERMSIG:     %d\n", WTERMSIG(status));
                printf("WSTOPSIG:     %d\n", WSTOPSIG(status));
            }
        }
}

/**
 * Executes a cmd
 * @param cmd - array of strings containing command, last element must be
 *              NULL
 * @param cmdSize - size of command array
 * @return none
 */
void execute(char **cmd, uint32_t cmdSize){
    
    if(DEBUG_SHELLCMD){//display commands to shell
        uint32_t i = 0;
        while(cmd[i] != NULL){ 
            printf(" %s", cmd[i]);
            i++;
        }
        printf("\n");
    }
    
    int returnVal = execvp(cmd[0], cmd);
    if(returnVal == -1) printf("Error executing %s", cmd[0]);
    printf("returnVal\r\n");

    
        exit(0); //kill child proc
}

char * checkRedirect(char **cmd, uint32_t cmdSize){
    uint32_t i = 0;
    while(cmd[i] != NULL){
        if(!strcmp(cmd[i],">")){
            return(cmd[i+1]);
        }
        i++;
    }
    return(NULL);
}

void executeRedirect(char **cmd, uint32_t cmdSize, char *redirectLoc){
        int fd; 
        int stdout_saved;
        pid_t child = 0;
        pid_t error = -1;
        if(DEBUG_SHELLCMD){printf("in executeRedirect\n\r");}
        
        if((fd = open(redirectLoc, O_RDWR|O_CREAT)) == -1){
             printf("Error open()\n\r");   
            }
        else{
            stdout_saved = dup(1); //save stdout
            dup2(fd,1); //redirect stdout to fd
            close(fd);
            pid_t pid = forkProc();
            if(pid != error){
                if(pid != child){ //parent process
                    if(DEBUG_SHELLCMD)printf("parent wait %d\n\r", getpid());
                    dup2(stdout_saved, 1); //return stdout
                    close(stdout_saved);
                    waitProc(pid); // wait for child process
                    if(DEBUG_SHELLCMD)printf("parent done wait\n\r");
                    exit(0);
                    }
                else { //child process
                    if(DEBUG_SHELLCMD)fprintf(stderr,"in child %d\n\r", getpid());
                    execute(cmd, cmdSize);
                    exit(1);
                    }

                }
            }
}

